import requests
from typing import Union, List


class Github:

    def __init__(self, repo: str = "django") -> None:
        self.project_id = "JscorpTech"
        self.repo = repo
        self.relase_urls = {
            "list": "releases",
            "latest": "releases/latest",
            "detail": "releases/tags/{}",
        }

    def request(self, action):
        url = "https://api.github.com/repos/{}/{}/{}".format(self.project_id, self.repo, action)
        response = requests.get(url)
        if response.status_code == 200:
            return response.json()
        raise Exception("Server bilan aloqa yo'q")

    def releases(self, version=None) -> Union[List[str]]:
        """Barcha releaselarni"""
        versions = list(map(lambda x: x["name"], self.request(self.relase_urls["list"])))
        if version:
            return self.check_version(version, versions)
        return versions

    def latest_release(self) -> Union[str]:
        """Oxirgi release"""
        return self.request(self.relase_urls["latest"])["name"]

    def check_version(self, version: Union[str], versions: Union[List[str]]):
        """Versionni tekshirish"""
        if version not in versions:
            raise Exception("{} mavjud emas boshqa versiya tanlang: {}".format(version, ", ".join(versions)))
        return versions

    def branches(self):
        response = []
        branches = list(map(lambda branch: branch["name"], self.request("branches")))
        for branch in branches:
            if str(branch).startswith("V") or branch == "main" or branch == "dev":
                response.append(branch)
        response.reverse()
        return response



import questionary.question
import os
from rich import print
import typer
from cookiecutter.main import cookiecutter
import questionary
from typing import Annotated
from .generate import Generate
from .utils import Jst
from .api import Github
from .translate import Translate
from .module import Module
from jst_aicommit.main import JstAiCommit

app = typer.Typer()

BASE_DIR = os.getcwd()


@app.command(name="install", help="Modul o'rnatish")
def install_module(
    module_name: Annotated[str, typer.Argument()] = None, version: str = typer.Option(None, "--version", "-v")
):
    Module().run(module_name, version)


@app.command(name="create", help="Yangi loyiha yaratish")
def create_project(version: str = typer.Option(None, "--version", "-v")):
    if version is None:
        version = Github().latest_release()
        print("version: ", version)
    else:
        Github().releases(version)
    template = questionary.text("Template: ", default="django").ask()
    if template == "django" or (template.startswith("http") is not True and not os.path.exists(template)):
        template = "https://github.com/JscorpTech/{}".format(template)
    choices = [
        "cacheops",
        "silk",
        "storage",
        "rosetta",
        "channels",
        "ckeditor",
    ]
    questions = {
        "project_name": {"type": "text", "message": "Project name: ", "default": "django"},
        "settings_module": {
            "type": "select",
            "message": "Settings file",
            "choices": [
                "config.settings.local",
                "config.settings.production",
            ],
        },
        "packages": {
            "type": "checkbox",
            "message": "O'rtailadigan kutubxonalarni tanlang",
            "choices": choices,
        },
        "runner": {
            "type": "select",
            "message": "Runner",
            "choices": ["wsgi", "asgi"],
        },
        "script": {
            "type": "select",
            "message": "Script file",
            "choices": ["entrypoint.sh", "entrypoint-server.sh"],
        },
        "key": {"type": "text", "default": "key", "message": "Django key"},
        "port": {"type": "text", "default": "8081", "message": "Port"},
        "phone": {"type": "text", "default": "998888112309", "message": "Default admin phone"},
        "password": {"type": "text", "default": "2309", "message": "Admin password"},
        "max_line_length": {"type": "text", "default": "120", "message": "Flake8 and black max line length"},
    }
    answers = {}
    for key, value in questions.items():
        method = value.pop("type")
        answers[key] = getattr(questionary, method)(**value).ask()
    answers["project_slug"] = answers["project_name"].lower().replace(" ", "_").replace("-", "_").replace(".", "_")
    packages = answers.pop("packages")
    context = {
        **{choice: choice in packages for choice in choices},
        **answers,
    }
    cookiecutter(
        template,
        checkout=version,
        no_input=True,
        extra_context=context,
    )


@app.command(name="generate", help="Compoment generatsiya qilish")
def generate():
    Generate().run()


@app.command(name="aic", help="O'zgarishlarga qarab atomatik git commit yaratadi")
def aic():
    JstAiCommit().run()


@app.command(name="init", help="jst.json config faylini yaratish")
def init():
    Jst().make_config()


@app.command(name="requirements", help="Kerakli kutubxonalar")
def requirements():
    Jst().requirements()


@app.command(name="translate", help="Avtomatik tarjima")
def translate():
    Translate().run()


if __name__ == "__main__":
    app()



import isort
from typing import Union
import black


class Code:
    def __init__(self) -> None:
        pass

    @staticmethod
    def format_code(file_path: Union[str]) -> None:
        """Black and Isort format code"""
        with open(file_path, "r") as file:
            code = black.format_str(
                isort.code(file.read(), config=isort.Config(profile="black", line_length=120)),
                mode=black.FileMode(line_length=120),
            )
        with open(file_path, "w") as file:
            file.write(code)



import zipfile
import tempfile
import requests
import os
from .api import Github
import questionary
import shutil
from uuid import uuid4
from typing import Union
from .utils import Jst
from rich import print


class Module:
    modules = {
        "default": "https://github.com/JscorpTech/module-default.git",
        "authbot": "https://github.com/JscorpTech/module-authbot.git",
        "websocket": "https://github.com/JscorpTech/module-websocket.git",
    }

    def __init__(self):
        self.config = Jst().load_config()

    def _subfolder_to_parent(self, path):
        """ichki papkadagi ko'dlarni parent papkaga ko'chirish"""
        extracted_file = os.path.join(path, os.listdir(path)[0])

        for file in os.listdir(extracted_file):
            shutil.move(os.path.join(extracted_file, file), path)
        os.rmdir(extracted_file)

    def _download(self, url, dir) -> Union[str]:
        """modulni yuklash"""
        file = os.path.join(dir, "%s.zip" % uuid4())
        with requests.get(url, stream=True) as response:
            response.raise_for_status()
            with open(file, "wb") as zip_file:
                for chunk in response.iter_content(chunk_size=8192):
                    zip_file.write(chunk)
        return file

    def _extract(self, module_name, zip_path):
        modules_dir = os.path.join(os.getcwd(), self.config["dirs"]["apps"])
        extract_dir = os.path.join(modules_dir, module_name)
        if os.path.exists(extract_dir):
            raise Exception("Modul mavjud")
        os.makedirs(extract_dir, exist_ok=True)

        with zipfile.ZipFile(zip_path, "r") as zip_ref:
            zip_ref.extractall(extract_dir)
        return extract_dir

    def _download_and_extract_module(self, module_name, url):
        with tempfile.TemporaryDirectory() as temp_dir:
            # Download the module
            zip_path = self._download(url, temp_dir)
            # Extract the module
            extract_dir = self._extract(module_name, zip_path)
            # Move the module to the correct location
            self._subfolder_to_parent(extract_dir)

            with open(os.path.join(extract_dir, "apps.py"), "r+") as file:
                data = file.read()
                file.seek(0)
                file.write(data.replace("{{module_name}}", "%s%s" % (self.config.get("apps", ""), module_name)))
                file.truncate()

    def run(self, module_name, version=None):
        module = questionary.select("Modulni tanlang", choices=self.modules.keys()).ask()
        api = Github("module-%s" % module)
        if module_name is None:
            module_name = module
        if version is None:
            version = api.latest_release()
        else:
            api.releases(version)
        print("[bold red]version: %s[/bold red]" % version)
        module = "https://github.com/JscorpTech/module-{}/archive/refs/tags/{}.zip".format(module, version)

        self._download_and_extract_module(module_name, module)
        print("[bold green]Modul o'rnatish yakunlandi[/bold green]")



from typing import List, Union, Optional, Generator
import questionary.question
import os
import questionary
from .utils import File, Code, Jst
from pathlib import Path
from os.path import join


class Generate:
    name: Optional[str] = None
    file_name: Optional[str] = None

    def __init__(self) -> None:
        self.config = Jst().load_config()
        dirs = self.config.get("dirs", {})
        self.path = {
            "apps": dirs.get("apps", "./core/apps/"),
            "model": dirs.get("models", "models/"),
            "serializer": dirs.get("serializers", "serializers/"),
            "view": dirs.get("views", "views/"),
            "permission": dirs.get("permissions", "permissions/"),
            "admin": dirs.get("admin", "admin/"),
            "test": dirs.get("tests", "tests/"),
            "translation": dirs.get("translation", "translation/"),
            "validator": dirs.get("validators", "validators/"),
            "form": dirs.get("forms", "forms/"),
            "filter": dirs.get("filters", "filters/"),
            "signal": dirs.get("signals", "signals/"),
            "stubs": join(os.path.dirname(__file__), "stubs"),
        }
        self.modules = [
            "model",
            "serializer",
            "view",
            "permission",
            "admin",
            "test",
            "translation",
            "validator",
            "form",
            "filter",
            "signal",
        ]
        self.stubs = {
            "init": "__init__.stub",
            "model": "model.stub",
            "serializer": "serializer.stub",
            "view": "view.stub",
            "permission": "permission.stub",
            "admin": "admin.stub",
            "test": "test.stub",
            "translation": "translation.stub",
            "validator": "validator.stub",
            "form": "form.stub",
            "filter": "filter.stub",
            "signal": "signal.stub",
        } | self.config.get("stubs", {})

    def _directory_ls(self, path: Union[str]) -> Generator[Union[Path], None, None]:
        """Directory items list"""
        ignore = ["logs"]
        for item in Path(path).iterdir():
            if item.name not in ignore and item.is_dir():
                yield item

    def _get_apps(self) -> Generator[Union[str], None, None]:
        """Django applar ro'yxatini qaytaradi"""
        dirs = self._directory_ls(self.path["apps"])
        for item in dirs:
            if item.joinpath("apps.py").exists():
                yield item.name

    def __get_stub_path(self, name):
        """Stubfayil manzilini olish"""
        if Path(self.stubs[name]).exists():
            return self.stubs[name]
        path = Path(self.path["stubs"], self.stubs[name])
        if path.exists():
            return path
        raise Exception("Stub fayil mavjud emas")

    def _read_stub(self, name: Union[str], append: Union[bool] = False) -> str:
        """Get stub"""
        response = ""
        top_content = ""
        with open(self.__get_stub_path(name)) as file:
            for chunk in file.readlines():
                if chunk.startswith("!!"):
                    top_content += chunk.replace("!!", "", 2)
                    continue
                elif append and chunk.startswith("##"):
                    continue
                elif not append and chunk.startswith("##"):
                    chunk = chunk.replace("##", "", 2)
                response += chunk
        if append:
            response = "\n" + response
        return top_content, response

    def _get_module_name(self, prefix: Union[str] = ""):
        return f"{str(self.name).capitalize()}{prefix}"

    def _write_file(
        self,
        file_path: Union[str],
        stub: Union[str],
        prefix: Union[str] = "",
        append: Union[bool] = False,
    ):
        if not os.path.exists(file_path):
            open(file_path, "w").close()
        with open(file_path, "r+") as file:
            file_content = file.read()
            top_content, content = self._read_stub(stub, append=append)
            file.seek(0)
            file.write(top_content.format(name_cap=self.name.capitalize(), file_name=self.file_name))
            file.write(file_content)
            file.write(
                content.format(
                    class_name=self._get_module_name(prefix),
                    name=self.name,
                    name_cap=self.name.capitalize(),
                )
            )

    def _import_init(self, init_path: Union[str], file_name: Union[str]):
        """__init__.py fayliga kerakli fayillarni import qiladi mavjud bo'lmasa yaratadi"""
        with open(init_path, "a") as file:
            file.write(self._read_stub("init")[1].format(file_name=file_name))
        Code.format_code(init_path)

    def make_folders(self, app: Union[str], modules: Union[List[str]]) -> bool:
        """Agar kerakli papkalar topilmasa yaratadi"""
        apps_dir = join(self.path["apps"], app)
        for module in modules:
            module_dir = join(apps_dir, self.path[module])
            file_path = join(module_dir, f"{self.file_name}.py")
            init_path = join(module_dir, "__init__.py")
            File.mkdir(module_dir)
            if module == "serializer":
                module_dir = join(module_dir, self.file_name)
                file_path = join(module_dir, f"{self.name}.py")
                File.mkdir(module_dir)
                self._import_init(join(module_dir, "__init__.py"), file_name=self.name)
            if not os.path.exists(file_path):
                self._import_init(init_path, self.file_name)
                self._write_file(file_path, module, module.capitalize())
            else:
                self._write_file(file_path, module, module.capitalize(), append=True)
            Code.format_code(file_path)
        return True

    def run(self) -> None:
        """Ishga tushurish uchun"""
        self.file_name = questionary.text("File Name: ").ask()
        self.name = questionary.text("Name: ").ask()

        app = questionary.select("Appni tanlang", choices=list(self._get_apps())).ask()
        modules = questionary.checkbox("Kerakli modullarni tanlang", self.modules).ask()
        self.make_folders(app, modules)
